import * as stream from 'stream';
import decryptChunk from '../decryption/decryptChunk';

export default function decryptStream(inputStream: stream.Readable, decryptionKeys: [number, number, number]): stream.Readable {
  let skippedRandomHeader = false;

  const outStream = new stream.Readable({
    read(size) {
      //There are 12 random bytes at the beginning, we need to use them to initialize the decryption keys, but we can ignore the decrypted bytes.
      if (!skippedRandomHeader) {
        const encryptedHeader: Buffer = inputStream.read(12);
        decryptChunk(encryptedHeader, decryptionKeys);
        skippedRandomHeader = true;
      }

      //Decrypt chunk
      const encryptedChunk: Buffer = inputStream.read(size);
      if (encryptedChunk === null) {
        //If end has been reached, stop
        this.push(null);
      } else {
        const decryptedChunk = decryptChunk(encryptedChunk, decryptionKeys);
        this.push(decryptedChunk);
      }
    },
  });

  inputStream.on('end', () => {
    outStream.emit('end');
  });

  return outStream;
}
